#define F_CPU 16000000

#include <avr/io.h>

#include <util/delay.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include "usart.h"

#define MIN_POW 185
#define MAX_POW 225

/*prag de culoare determiant  prin incercari*/
#define THRESHOLD 400

void pwm_init(void) {

	/*
	 * setare pwm
	 */
	TCCR0A |= (1 << COM0A1) | (1 << COM0B1) | (1 << WGM01) | (1 << WGM00);
	TCCR0B |= (1 << CS00);
	TIMSK0 = 0;

	/* seeteaza motoarele la ruratia minima*/	
	OCR0A = MIN_POW;
	OCR0B = MIN_POW;
}


void ADC_init(void)
{
	/* initalizare ADC */
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	/* portneste adc
	 * asteapta conversia sa se termine si returneaza valoarea
	 */
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

return ADC;
}

/*
 * fucntie fol;osita pentru a citi valoarea senzorilor
 */
void debug_usart(void)

{
int i, val;
char str[10];

while(1)

{

	for (i = 0; i < 8; i++) {
		val = ADC_get(i);
    	itoa(val, str, 10);
    	USART0_print(str);
    	USART0_print(" ");
	}
	USART0_print("\n\n\n\n");
	_delay_ms(3000);

}

}

void run_linefollower(void) {

int move = 0;
int last_move = 0;
int i, on_track;

int x[8] = {-1, -1, -1, -1, 1, 1, 1, 1};

while (1) {

	move = 0;
	on_track = 0;
	
	for (i = 0; i < 8; i++) {
		/*veridica daca vreun senzor se afla pe linia neagra*/
		if (ADC_get(i) > THRESHOLD) {
			move += x[i];
			on_track = 1;	
		}
		
	}
	
	/*
	 * daca a pierdut linia copiaza ultima mutare pana gaseste linia iar
	 */
	if(on_track == 0) {
		move = last_move;
	}


	if (move < 0) {
		/* Senzorii din dreapta sunt pe linie
		 * inseamna ca trebuie virat spre dreapta pentru a centra senzorul
		 * incetineste motorul drept si il accelereaza pe cel stang
		 */
		OCR0A = MAX_POW;
		OCR0B = MIN_POW - 15;	

	} else if (move > 0) {
		/* Senzorii din stanga sunt pe linie
		 * inseamna ca trebuie virat spre stanga pentru a centra senzorul
		 * incetineste motorul stang si il accelereaza pe cel drept
		 */
		OCR0A = MIN_POW - 15;
		OCR0B = MAX_POW;
	
	} else {
		/*
		 * linia e la mijloc, continua deplasarea constant
		 */
		OCR0A = MIN_POW;
		OCR0B = MIN_POW;
	}
	
	/*
	 * actualizeaza ultima miscare
	 */ 
	last_move = move;
	}
}

void ports_init(void) {
	
	PORTB = 0;
	DDRA = 0;
	PORTA = 0xFF;

	/*setez pinii de directie si cei de pwm*/
	DDRB |= (1 << PB1) | (0 << PB2);
	DDRB |= (1 << PB3) | (1 << PB4);
	DDRB |= (1 << PB5) | (0 << PB6);
	
	/*dau directia de rotatie a motoarelor*/
	PORTB |= (1 << PB6);
	PORTB |= (1 << PB1);
}

int main() {
	
	ports_init();
	

	pwm_init();
	ADC_init();
    
    /*folosit pentru debug*/
    //USART0_init();
	//debug_usart();
	
	run_linefollower();

	return 0;

}